@@ -581,10 +581,10 @@ bcp_control(DBPROCESS * dbproc, int field, DBINT value)
 /*
  * \ingroup dblib_bcp
  * \brief Get BCP batch option
- * 
+ *
  * \param dbproc contains all information needed by db-lib to manage communications with the server.
- * \remarks This function is specific to FreeTDS.  
- * 
+ * \remarks This function is specific to FreeTDS.
+ *
  * \return the value that was set by bcp_control.
  * \sa 	bcp_batch(), bcp_control()
  */
@@ -596,6 +596,28 @@ bcp_getbatchsize(DBPROCESS * dbproc)
 	return dbproc->hostfileinfo->batch;
 }
 
+/*
+ * \ingroup dblib_bcp
+ * \brief Get number of columns in the BCP data file
+ *
+ * \param dbproc contains all information needed by db-lib to manage communications with the server.
+ * \remarks This function is specific to FreeTDS.
+ *
+ * \return the value that was set by bcp_columns(). This represents the number
+ *     of columns of data in the saved data file. Usually the same as the number
+ *     of columns in the database table, but could be fewer if a Format File is
+ *     in use which only selects a subset of the table columns.
+ *
+ * \sa bcp_columns()
+ */
+int
+bcp_gethostcolcount(DBPROCESS *dbproc)
+{
+	CHECK_CONN(-1);
+	CHECK_PARAMETER(dbproc->hostfileinfo, SYBEBCPI, -1);
+	return dbproc->hostfileinfo->host_colcount;
+}
+
 /** 
  * \ingroup dblib_bcp
  * \brief Set "hints" for uploading a file.  A FreeTDS-only function.  
@@ -1053,11 +1075,12 @@ _bcp_check_eof(DBPROCESS * dbproc, FILE *file, int icol)
  */
 static TDSRET
 _bcp_convert_in(DBPROCESS *dbproc, TDS_SERVER_TYPE srctype, const TDS_CHAR *src, TDS_UINT srclen,
-		TDS_SERVER_TYPE desttype, BCPCOLDATA *coldata)
+		TDS_SERVER_TYPE desttype, TDSCOLUMN *col)
 {
 	bool variable = true;
 	CONV_RESULT cr, *p_cr;
 	TDS_INT len;
+	BCPCOLDATA *coldata = col->bcp_column_data;
 
 	coldata->is_null = false;
 
@@ -1076,6 +1099,10 @@ _bcp_convert_in(DBPROCESS *dbproc, TDS_SERVER_TYPE srctype, const TDS_CHAR *src,
 
 	coldata->datalen = len;
 	if (variable) {
+		if (len > col->on_server.column_size) {
+			dbperror(dbproc, SYBECOFL, 0);
+			return TDS_FAIL;
+		}
 		free(coldata->data);
 		coldata->data = (TDS_UCHAR *) cr.c;
 	}
@@ -1133,7 +1160,8 @@ rtrim_bcpcol(TDSCOLUMN *bcpcol)
  *
  * \param dbproc contains all information needed by db-lib to manage communications with the server.
  * \param hostfile 
- * \param row_error 
+ * \param row_error set to true if an error is detected in the row
+ * \param skip skip the row
  * 
  * \return MORE_ROWS, NO_MORE_ROWS, or FAIL.
  * \sa 	BCP_SETL(), bcp_batch(), bcp_bind(), bcp_colfmt(), bcp_colfmt_ps(), bcp_collen(), bcp_colptr(), bcp_columns(), bcp_control(), bcp_done(), bcp_exec(), bcp_getl(), bcp_init(), bcp_moretext(), bcp_options(), bcp_readfmt(), bcp_sendrow()
@@ -1336,7 +1364,7 @@ _bcp_read_hostfile(DBPROCESS * dbproc, FILE * hostfile, bool *row_error, bool sk
 				desttype = tds_get_conversion_type(bcpcol->column_type, bcpcol->column_size);
 
 				rc = _bcp_convert_in(dbproc, hostcol->datatype, (const TDS_CHAR*) coldata, collen,
-						     desttype, bcpcol->bcp_column_data);
+						     desttype, bcpcol);
 				if (TDS_FAILED(rc)) {
 					hostcol->column_error = HOST_COL_CONV_ERROR;
 					*row_error = true;
@@ -1526,6 +1554,7 @@ _bcp_exec_in(DBPROCESS * dbproc, DBINT * rows_copied)
 
 				while (error_row_size > 0) {
 					size_t chunk = TDS_MIN((size_t) error_row_size, chunk_size);
+					size_t written;
 
 					if (!row_in_error) {
 						if ((row_in_error = tds_new(char, chunk)) == NULL) {
@@ -1533,13 +1562,13 @@ _bcp_exec_in(DBPROCESS * dbproc, DBINT * rows_copied)
 						}
 					}
 
-					if (fread(row_in_error, chunk, 1, hostfile) != 1) {
+					if (fread(row_in_error, 1, chunk, hostfile) != chunk)
 						tdsdump_log(TDS_DBG_ERROR, "BILL fread failed after fseek\n");
-					}
-					count = (int)fwrite(row_in_error, chunk, 1, errfile);
-					if( (size_t)count < chunk ) {
+
+					written = fwrite(row_in_error, 1, chunk, errfile);
+					if (written < chunk)
 						dbperror(dbproc, SYBEBWEF, errno);
-					}
+
 					error_row_size -= chunk;
 				}
 				free(row_in_error);
@@ -2257,8 +2286,7 @@ _bcp_get_col_data(TDSBCPINFO *bcpinfo, TDSCOLUMN *bindcol, int offset TDS_UNUSED
 	if (collen < 0)
 		collen = (int) strlen((char *) dataptr);
 
-	rc = _bcp_convert_in(dbproc, coltype, (const TDS_CHAR*) dataptr, collen,
-					    desttype, bindcol->bcp_column_data);
+	rc = _bcp_convert_in(dbproc, coltype, (const TDS_CHAR *) dataptr, collen, desttype, bindcol);
 	if (TDS_FAILED(rc))
 		return rc;
 	rtrim_bcpcol(bindcol);
